package xylia.sax.contentmodelparser;

import java.io.* ;
import java.util.Vector;
import javax.xml.parsers.* ;

import junit.framework.*;

import org.xml.sax.* ;
import org.xml.sax.ext.* ;

/** Some unit tests for the contentmodelparser package.
 *  
 * Copyright:    Copyright (c) 2001
 * Company:      MUN
 * @author Theodore S. Norvell
 * @version 1.0
 */

public class ContentParserTests extends TestCase
                                implements ErrorHandler {

    public ContentParserTests(String str) { super(str) ; }

    public ContentParserTests() { this("ContentParser Tests" ) ; }

    ContentModelMap cmm ;

    protected void setUp() {

        cmm = new ContentModelMap() ;

        try {
            SAXParserFactory spf = SAXParserFactory.newInstance() ;
            spf.setValidating(true);
            spf.setNamespaceAware(false);
            SAXParser sp = spf.newSAXParser() ;
            XMLReader xmlr = sp.getXMLReader() ;

            DeclHandler cmpdh = new ContentModelParserDeclHandler( cmm ) ;
            xmlr.setProperty("http://xml.org/sax/properties/declaration-handler",
                              cmpdh );
            xmlr.setErrorHandler( this ) ;

            Reader reader = new FileReader( "contentparsetest.xml" ) ;
            InputSource is = new InputSource( reader ) ;

            xmlr.parse( is ) ; }
        catch( SAXException e ) {
            System.out.println(e.getMessage()) ;
            throw new Error( e.getMessage() ) ; }
        catch( ParserConfigurationException e ) {
            System.out.println(e.getMessage()) ;
            throw new Error( e.getMessage() ) ; }
        catch( FileNotFoundException e ) {
            System.out.println(e.getMessage()) ;
            throw new Error( e.getMessage() ) ; }
        catch( IOException e ) {
            System.out.println(e.getMessage()) ;
            throw new Error( e.getMessage() ) ; }
    }

    public void warning(SAXParseException p0) throws SAXException {
        System.out.println("Warning: "+p0.getMessage() ); }

    public void error(SAXParseException p0) throws SAXException {
        System.out.println("Error: "+p0.getMessage() );
        throw p0 ; }

    public void fatalError(SAXParseException p0) throws SAXException {
        System.out.println("Fatal Error: "+p0.getMessage() );
        throw p0 ; }

    public void testSetUp() {
    }

    public void test0() {
        assertTrue( cmm.get("p") != null ) ;
        assertTrue( cmm.get("p") instanceof DefaultContentModel ) ;
        DefaultContentModel p = (DefaultContentModel) cmm.get("p") ;
        FiniteStateAutomaton fsa = p.getFSA() ;

        Vector v0 = new Vector() ;
        v0.addElement( "em" ) ;
        v0.addElement( "strong" ) ;
        v0.addElement( "#PCDATA" ) ;
        v0.addElement( "em" ) ;
        v0.addElement( "em" ) ;

        assertTrue( fsa.match(v0) ) ;

        v0.addElement( "p" );

        assertTrue( !fsa.match(v0) ) ;
    }

    public void test1() {
        assertTrue( cmm.get("subderivation") != null ) ;
        assertTrue( cmm.get("subderivation") instanceof DefaultContentModel ) ;
        DefaultContentModel p = (DefaultContentModel) cmm.get("subderivation") ;
        FiniteStateAutomaton fsa = p.getFSA() ;

        Vector v0 = new Vector() ;
        v0.addElement( "subject" ) ;

        assertTrue( fsa.match(v0) ) ;

        v0.addElement( "step" );

        assertTrue( fsa.match(v0) ) ;

        v0.addElement( "interstep" );

        assertTrue( ! fsa.match(v0) ) ;

        v0.addElement( "step" );

        assertTrue( fsa.match(v0) ) ;

        v0.addElement( "interstep" );

        assertTrue( ! fsa.match(v0) ) ;

        v0.addElement( "step" );

        assertTrue( fsa.match(v0) ) ;

        v0.add( 2, "interstep" );

        assertTrue( !fsa.match(v0) ) ;

        v0.remove(2);

        assertTrue( fsa.match(v0) ) ;

        v0.remove(2);
        v0.add(2,"#PCDATA") ;

        assertTrue( ! fsa.match(v0) ) ;
    }
    
    public void testConversion() {
        // The results of this subroutine must be chacked by hand!
        /// a
        RegExp p0 = new RegExp(RegExp.NAME, "a") ; 
        System.out.println( "Converting regexp "+p0 ) ;
        FiniteStateAutomaton fsa = ConvertToDFA.convertToDFA( p0 ) ;
        System.out.println( "Result is: "+fsa ) ;
        
        // a | b
        RegExp p1 = new RegExp( RegExp.NAME, "b" ) ;
        RegExp p2 = new RegExp( RegExp.ALT, p0, p1 ) ;
        System.out.println( "Converting regexp "+p2 ) ;
        fsa = ConvertToDFA.convertToDFA( p2 ) ;
        System.out.println( "Result is: "+fsa ) ;
        
        // a b
        RegExp p3 = new RegExp( RegExp.SEQ, p0, p1 ) ;
        System.out.println( "Converting regexp "+p3 ) ;
        fsa = ConvertToDFA.convertToDFA( p3 ) ;
        System.out.println( "Result is: "+fsa ) ;
        
        // (a b)+
        RegExp p4 = new RegExp( RegExp.KPLUS, p3 ) ;
        System.out.println( "Converting regexp "+p4 ) ;
        fsa = ConvertToDFA.convertToDFA( p4 ) ;
        System.out.println( "Result is: "+fsa ) ;
        
        // (a b)*
        RegExp p5 = new RegExp( RegExp.KSTAR, p3 ) ;
        System.out.println( "Converting regexp "+p5 ) ;
        fsa = ConvertToDFA.convertToDFA( p5 ) ;
        System.out.println( "Result is: "+fsa ) ;
        
        // (a b)?
        RegExp p6 = new RegExp( RegExp.OPT, p3 ) ;
        System.out.println( "Converting regexp "+p6 ) ;
        fsa = ConvertToDFA.convertToDFA( p6 ) ;
        System.out.println( "Result is: "+fsa ) ;
        
        // (a b+ c | a b? d? c)*
        RegExp r0 = new RegExp( RegExp.NAME, "a" ) ,
               r1 = new RegExp( RegExp.NAME, "b" ),
               r2 = new RegExp( RegExp.KPLUS, r1 ),
               r3 = new RegExp( RegExp.NAME, "c" ),
               r4 = new RegExp( RegExp.SEQ, r0, new RegExp(RegExp.SEQ, r2, r3 ) ) ,
               r5 = new RegExp( RegExp.NAME, "a" ),
               r6 = new RegExp( RegExp.OPT, new RegExp( RegExp.NAME, "b" ) ),
               r7 = new RegExp( RegExp.NAME, "d" ),
               r8 = new RegExp( RegExp.SEQ, r5, new RegExp( RegExp.SEQ, r6, r7 ) ),
               r9 = new RegExp( RegExp.KSTAR, new RegExp( RegExp.ALT, r4, r8 ) ) ;
               
        System.out.println( "Converting regexp "+r9 ) ;
        fsa = ConvertToDFA.convertToDFA( r9 ) ;
        System.out.println( "Result is: "+fsa ) ;
    }
}