package xylia.sax.contentmodelparser;

import java.util.Vector;

import xylia.utilities.Assert;
/** A tree node for representing regular expressions.
 * Title:        Xylia
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      Memorial
 * @author Theodore Norvell, Dennis Peters and others.
 * @version 1.0
 */

public class RegExp {

    public static final int PCDATA = 0, NAME = 1, ALT = 2, SEQ = 3, OPT = 4, KSTAR = 5, KPLUS = 6 ;

    private int kind ;

    private String name ;

    private RegExp left = null, right = null ;

    public RegExp(int kind) {
        Assert.assert( kind == PCDATA ) ;
        this.kind = kind ; }

    public RegExp(int kind, String name ) {
        Assert.assert(  kind == NAME ) ;
        this.kind = kind ;
        this.name = name ; }

    public RegExp(int kind, RegExp operand) {
        Assert.assert( kind == OPT || kind == KSTAR || kind == KPLUS ) ;
        this.kind = kind ;
        this.left = operand ; }

    public RegExp(int kind, RegExp left, RegExp right ) {
        Assert.assert( kind == ALT || kind == SEQ ) ;
        this.kind = kind ;
        this.left = left ;
        this.right = right ; }

    public int getKind() { return kind ; }

    public RegExp getOperand( int i ) {
        switch( i ) {
        case 0 : {
            Assert.assert( kind == OPT || kind == KSTAR || kind == KPLUS
                        || kind == ALT || kind == SEQ ) ;
            return left ; }
        case 1 : {
            Assert.assert( kind == ALT || kind == SEQ ) ;
            return right ; }
        default : {
            Assert.assert( false ) ;
            return null ; } }
    }

    public String getName() {
        Assert.assert( kind == NAME ) ;
        return name ;
    }
    
    public String toString() {
        switch( kind ) {
            case PCDATA : return "#PCDATA" ;
            case NAME : return name ;
            case ALT : return "("+getOperand(0)+" | "+getOperand(1)+")" ;
            case SEQ : return getOperand(0)+" "+getOperand(1) ;
            case OPT :   return "("+getOperand(0)+")?" ;
            case KSTAR : return "("+getOperand(0)+")*" ;
            case KPLUS : return "("+getOperand(0)+")+" ;
            default: Assert.assert(false) ; return null ;
        }
    }
}